// Game.cpp: implementation of the CGame class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <MATH.H>
#include <MMSYSTEM.H>
#include "glDuke.h"
#include "Game.h"
#include "Settings.h"
#include "glDukeControls.h"
#include "ScriptManager.h"
#include "Movie.h"
#include "CtrlGame.h"
#include "CtrlConsole.h"
#include "ModTC.h"
#include "ModelDef.h"

//#undef DISPLAYENEMIES

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern CSettings set;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGame::CGame()
{
	nCheckKey = 0;
	nUserLevel = 0;
	nHealth = -1;
	nConsole = 0;
	memset(nWeaponsAmmo, 0, sizeof(nWeaponsAmmo));
}

CGame::~CGame()
{

}

void CALLBACK TextDelay(UINT idEvent, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	XBUILDDISPLAYLIST *l = (XBUILDDISPLAYLIST*)dwUser;
	l->nElementCount = STATUS_BASE_ELEMENT_COUNT;
	XBUILD_GFX_SetDisplayList(XBUILD_GFX_LAYER_STATUS, l);
}

#define DUKE_NUMFONTBASE		2472
#define DUKE_STATUSFONTBASE		2821
#define DUKE_HUDSMALLFONTBASE	3010		//0

void CGame::ChangeHealth(int nNewHealth)
{
	char strTemp[16];
	if(nNewHealth == nHealth)
	{
		//Don't change for nothing
		return;
	}
	nHealth = nNewHealth;
	strcpy(strTemp, "");
	sprintf(strTemp, "%0.3i", nHealth);
	XBUILD_GFX_AddTileToDisplayList((strTemp[0] - 0x30) + DUKE_NUMFONTBASE, 18, 184, &m_Status, STATUS_BASE_UI_COUNT + 0);
	XBUILD_GFX_AddTileToDisplayList((strTemp[1] - 0x30) + DUKE_NUMFONTBASE, 26, 184, &m_Status, STATUS_BASE_UI_COUNT + 1);
	XBUILD_GFX_AddTileToDisplayList((strTemp[2] - 0x30) + DUKE_NUMFONTBASE, 34, 184, &m_Status, STATUS_BASE_UI_COUNT + 2);
	XBUILD_GFX_SetDisplayList(XBUILD_GFX_LAYER_STATUS, &m_Status);
}

void CGame::PrintTextOnStatus(char *lpstrText)
{
	//Status Font Base - 2821
	int nLenght;
	int i;
	int nX;
	int nTile;
	XBUILD_ART_TILEDATA *pTile;
	nLenght = strlen(lpstrText);
	nX = (320 - (nLenght * 8)) / 2;
	timeKillEvent(m_nStatusTimer);
	m_Status.nElementCount = STATUS_BASE_ELEMENT_COUNT;
	pTile = new XBUILD_ART_TILEDATA;
	for(i = 0; i < nLenght; i++)
	{
		if(lpstrText[i] != 0x20)
		{
			XBUILDDISPLAYELEMENT_TILE DTile;
			m_Status.nElementCount++;
			nTile = DUKE_STATUSFONTBASE + (toupper(lpstrText[i]) - 0x20);
			XBUILD_ART_GetTileData(nTile, pTile);
			DTile.dwTile = nTile;
			DTile.dwX = nX;
			DTile.dwY = 20;
			memcpy(m_Status.Element[m_Status.nElementCount - 1].pData, &DTile, sizeof(XBUILDDISPLAYELEMENT_TILE));
			m_Status.Element[m_Status.nElementCount - 1].dwType = XBUILDDISPLAYLIST_TYPE_TILE;
			nX += pTile->Width;
		}
		else
		{
			nX += 8;
		}
	}
	delete pTile;
	XBUILD_GFX_SetDisplayList(XBUILD_GFX_LAYER_STATUS, &m_Status);
	m_nStatusTimer = timeSetEvent(2000, 0, TextDelay, (DWORD)&m_Status, TIME_ONESHOT);
}

void CGame::TakeScreenShot()
{
	int i;
	FILE *f;
	char fname[32];
	char prompt[64];
	i = 0;
	while(1)
	{
		sprintf(fname, "SCR%0.5i.TGA", i);
		f = fopen(fname, "rb");
		if(f == NULL)
		{
			//We got a good filename
			break;
		}
		i++;
	}
	XBUILD_GFX_TakeScreenShot(fname);
	sprintf(prompt, "Saved screen shot as '%s'...", fname);
	PrintTextOnStatus(prompt);
}

void CGame::SetupMod()
{
	DUKENUKEMMOD m;
	FILE *stream;
	char strDir[64];
	char strExt[64];
	char strFile[64];
	char strDrive[64];
	char strSearch[128];
	if(!stricmp(set.m_Setup.lpstrModPath, "(NORMAL DUKE)"))
	{
		//Normal DukeNukem 3D
		strcpy(strCON, "GAME.CON");
		strcpy(strGRP, "DUKE3D.GRP");
		return;
	}
	_splitpath(set.m_Setup.lpstrModPath, strDrive, strDir, strFile, strExt);
	strcpy(strSearch, strDrive);
	strcat(strSearch, strDir);
	stream = fopen(set.m_Setup.lpstrModPath, "rb");
	fread(&m, sizeof(DUKENUKEMMOD), 1, stream);
	fclose(stream);
	strcpy(strCON, m.strNewCON);
	stream = fopen(m.strNewGRP, "rb");
	if(!stream)
	{
		//File isn't in directory
		strcpy(strGRP, strSearch);
		strcat(strGRP, m.strNewGRP);
	}
	else
	{
		strcpy(strGRP, m.strNewGRP);
	}
	XBUILD_GRPMANAGER_SetSecondSearchDir(strSearch);	
}

void CGame::PlayCorrectMusic()
{
	int nEpisode;
	int nLevel;
	sscanf(m_Level.GetBuffer(256), "e%il%i", &nEpisode, &nLevel);
	sscanf(m_Level.GetBuffer(256), "E%iL%i", &nEpisode, &nLevel);
	XBUILD_MUSIC_Play(c->Music[nEpisode].strFilename01 + ((nLevel - 1) * 32), set.m_Setup.nMusicDriver);
}

int CGame::IsBuiltInLevel()
{
	int nEpisode = -1;
	int nLevel = -1;
	sscanf(m_Level.GetBuffer(256), "e%il%i", &nEpisode, &nLevel);
	sscanf(m_Level.GetBuffer(256), "E%iL%i", &nEpisode, &nLevel);
	if(nEpisode == -1 || nLevel == -1)
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

int CGame::ActivateSectorEffector(int nSprite, int nSector)
{
	switch(pMap->Sprite[nSprite].LoTag)
	{
	case 7:
		//SE 7 : Teleport
		if(pMap->Sprite[nSprite].Palette <= 0)
		{
			XBUILD_SOUND_Play(c->DefineSound[GetCorrectSoundNumber(70)].strFilename, 0);
		}
		nSprite = XBUILD_ANIMATE_LookForSpriteWithSameTag(7, pMap->Sprite[nSprite].HiTag, nSprite);
		X = pMap->Sprite[nSprite].X;
		Z = pMap->Sprite[nSprite].Y;
		Y = pMap->Sprite[nSprite].Z;
		YAngle += 1024;
		XBUILD_ANIMATE_ChangeCurrentSector(pMap->Sprite[nSprite].SectorNum);
		return nSprite;
	}
	return -1;
}

void CGame__ST1617_Callback(void *pData)
{
	//We'd need to force a sound channel in SOUND_Play here
	GAME_SECTORSTATUS *pSS;
	CGame *c;
	int nsprt;
	pSS = (GAME_SECTORSTATUS*)pData;
	c = (CGame*)pSS->pGame;
	nsprt = XBUILD_ANIMATE_LookForSpriteInSector(pSS->nSector, 5, -1);
	if(nsprt != -1)
	{
		XBUILD_SOUND_Play(c->c->DefineSound[c->GetCorrectSoundNumber(c->pMap->Sprite[nsprt].LoTag)].strFilename, 0);
	}
}

int CGame::ActivateSectorTag(int nSector, int nUser)
{
	int nLoTag, nsprt, nNoSound, nSector2, nDist;
	__int32 *pValue;
	int nNegative = 1;
	nLoTag = pMap->Sector[nSector].LoTag;
	if(nLoTag == 0)
	{
		return -1;
	}
	nNoSound = 0;
	switch(nLoTag)
	{
	case 16:
		//ST16 : Elevator Platform Down
		if(!XBUILD_INCREMENTATOR_IsIncrementDead(SectorStatus[nSector].nIncrementator) && SectorStatus[nSector].nIncrementator != -1)
		{
			return -1;
		}
		if(SectorStatus[nSector].bActivated & 1)
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 0); 
			if(nSector2 == -1)
			{	
				//Sector wasn't found
				nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 1); 
				nNegative = -1;
			}
			pValue = &(pMap->Sector[nSector].FZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, nNegative * -4, pMap->Sector[nSector2].FZ, CGame__ST1617_Callback, (void*)&SectorStatus[nSector]);
			SectorStatus[nSector].bActivated |= 1;
		}
		else
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 1); 
			if(nSector2 == -1)
			{	
				//Sector wasn't found
				nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 0); 
				nNegative = -1;
			}
			pValue = &(pMap->Sector[nSector].FZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, nNegative * 4, pMap->Sector[nSector2].FZ, CGame__ST1617_Callback, (void*)&SectorStatus[nSector]);
			SectorStatus[nSector].bActivated &= ~1;
		}
		break;
	case 17:
		//ST17 : Elevator Platform Up
		if(!XBUILD_INCREMENTATOR_IsIncrementDead(SectorStatus[nSector].nIncrementator) && SectorStatus[nSector].nIncrementator != -1)
		{
			return -1;
		}
		if(SectorStatus[nSector].bActivated & 1)
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 1); 
			if(nSector2 == -1)
			{	
				//Sector wasn't found
				nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 0); 
				nNegative = -1;
			}
			pValue = &(pMap->Sector[nSector].FZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, nNegative * 4, pMap->Sector[nSector2].FZ, CGame__ST1617_Callback, (void*)&SectorStatus[nSector]);
			SectorStatus[nSector].bActivated &= ~1;
		}
		else
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 0); 
			if(nSector2 == -1)
			{	
				//Sector wasn't found
				nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 1); 
				nNegative = -1;
			}
			pValue = &(pMap->Sector[nSector].FZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, nNegative * -4, pMap->Sector[nSector2].FZ, CGame__ST1617_Callback, (void*)&SectorStatus[nSector]);
			SectorStatus[nSector].bActivated |= 1;
		}
		break;
	case 18:
		//ST18 : Elevator Down
		if(!XBUILD_INCREMENTATOR_IsIncrementDead(SectorStatus[nSector].nIncrementator) && SectorStatus[nSector].nIncrementator != -1)
		{
			return -1;
		}
		if(SectorStatus[nSector].bActivated & 1)
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 0); 
			nDist = pMap->Sector[nSector2].FZ - pMap->Sector[nSector].FZ;
			pValue = &(pMap->Sector[nSector].FZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, -4, pMap->Sector[nSector2].FZ, NULL, NULL);
			SectorStatus[nSector].bActivated &= ~1;
			pValue = &(pMap->Sector[nSector].CZ);
			XBUILD_INCREMENTATOR_Add((int*)pValue, -4, pMap->Sector[nSector].CZ + nDist, NULL, NULL);
		}
		else
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].FZ, 1, 1); 
			nDist = pMap->Sector[nSector2].FZ- pMap->Sector[nSector].FZ;
			pValue = &(pMap->Sector[nSector].FZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, 4, pMap->Sector[nSector2].FZ, NULL, NULL);
			SectorStatus[nSector].bActivated |= 1;
			pValue = &(pMap->Sector[nSector].CZ);
			XBUILD_INCREMENTATOR_Add((int*)pValue, 4, pMap->Sector[nSector].CZ + nDist, NULL, NULL);
		}
		break;
	case 20:
		//ST20 : Ceiling Door
		if(!XBUILD_INCREMENTATOR_IsIncrementDead(SectorStatus[nSector].nIncrementator) && SectorStatus[nSector].nIncrementator != -1)
		{
			return -1;
		}
		if(SectorStatus[nSector].bActivated & 1)
		{
			pValue = &(pMap->Sector[nSector].CZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, 4, pMap->Sector[nSector].FZ, NULL, NULL);
			SectorStatus[nSector].bActivated &= ~1;
		}
		else
		{
			nSector2 = XBUILD_ANIMATE_NextSectorNeighborZ(nSector, pMap->Sector[nSector].CZ, 0, 0);
			pValue = &(pMap->Sector[nSector].CZ);
			SectorStatus[nSector].nIncrementator = XBUILD_INCREMENTATOR_Add((int*)pValue, -4, pMap->Sector[nSector2].CZ, NULL, NULL);
			SectorStatus[nSector].bActivated |= 1;
		}
		break;
	case 23:
		//ST23 : Swing Door
		long nX2, nY2;
		int i;
		short ang;
		XBUILD_MAP_WALL *w, *w2;
		nsprt = XBUILD_ANIMATE_LookForSpriteInSector(nSector, 1, 11);
		if(SectorStatus[nSector].bActivated & 1)
		{
			if(pMap->Sprite[nsprt].Angle > 1024)
			{
				ang = 512;
			}
			else
			{
				ang = -512;
			}
			SectorStatus[nSector].bActivated &= ~1;
		}
		else
		{
			if(pMap->Sprite[nsprt].Angle > 1024)
			{
				ang = -512;
			}
			else
			{
				ang = 512;
			}
			SectorStatus[nSector].bActivated |= 1;
		}
		w = &pMap->Wall[pMap->Sector[nSector].WallPointer];
		for(i = 0; i < pMap->Sector[nSector].WallNumber; i++, w++)
		{
			XBUILD_ANIMATE_RotatePoint(pMap->Sprite[nsprt].X, pMap->Sprite[nsprt].Y, w->X, w->Y, ang, &nX2, &nY2);
			w->X = nX2;
			w->Y = nY2;
			if(w->NextWall != 0xFFFF)
			{
				w2 = &pMap->Wall[w->NextWall];
				XBUILD_ANIMATE_RotatePoint(pMap->Sprite[nsprt].X, pMap->Sprite[nsprt].Y, w2->X, w2->Y, ang, &nX2, &nY2);
				w2->X = nX2;
				w2->Y = nY2;
			}
		}
		if(nUser != 1)
		{
			nsprt = XBUILD_ANIMATE_LookForSpriteWithSameTag(11, pMap->Sprite[nsprt].HiTag, nsprt);
			if(nsprt != -1)
			{
				ActivateSectorTag(pMap->Sprite[nsprt].SectorNum, 1);
			}
		}
		else
		{
			nNoSound = 1;
		}
		break;
	}
	if(!nNoSound)
	{
		nsprt = XBUILD_ANIMATE_LookForSpriteInSector(nSector, 5, -1);
		if(nsprt != -1)
		{
			XBUILD_SOUND_Play(c->DefineSound[GetCorrectSoundNumber(pMap->Sprite[nsprt].LoTag)].strFilename, 0);
		}
	}
	return -1;
}

void CGame::ProcessSectorTags()
{
	int nSector;
	int nSE;
	int nDZ;
	static int nLastTeleport = -1;
	unsigned int nWaterLatch = 0;
	nSector = XBUILD_ANIMATE_GetCurrentSector();
	switch(pMap->Sector[nSector].LoTag)
	{
	case 0:
		//SE 7 : Teleport
		nSE = XBUILD_ANIMATE_LookForSpriteInSector(nSector, 1, 7);
		if(nSE != -1)
		{
			nDZ = abs(pMap->Sprite[nSE].Z - Y);
			if(nDZ > 600)
			{	
				break;
			}
			if(nLastTeleport == nSector)
			{
				return;
			}
			nSE = ActivateSectorEffector(nSE, nSector);
			nLastTeleport = pMap->Sprite[nSE].SectorNum;
			return;
		}
		break;
	case 1:
		//ST 1 : Water
		//Y += 200;
		break;
	case 2:
		//ST 2 : Underwater
		nUnderwater = 1;
		return;
	case 32767:
		//ST 32767 : Secret Room
		//Display the 'You've found a secret!' message
		//And add a secret to your secret count
		break;
	case 65535:
		//ST 65535 : End of Level
		//Should terminate the level, but that's not implemented yet
		break;
	default:
		if(pMap->Sector[nSector].LoTag > 10000)
		{
			if(!(SectorStatus[nSector].bActivated & 1))
			{
				XBUILD_SOUND_Play(c->DefineSound[GetCorrectSoundNumber(pMap->Sector[nSector].LoTag - 10000)].strFilename, 0);
				SectorStatus[nSector].bActivated |= 1;
			}
		}
		break;
	}
	nLastTeleport = -1;
	nUnderwater = 0;
}

void CGame::UpdateAmbientSounds()
{
	int i, j;
	float fDistance;
	for(j = 0; j < nAmbiantCount; j++)
	{
		i = nAmbiantsSprites[j];
		fDistance = (float)sqrt(((float)(X - pMap->Sprite[i].X) * (float)(X - pMap->Sprite[i].X)) +  ((float)(Z - pMap->Sprite[i].Y) * (float)(Z - pMap->Sprite[i].Y)));
		if(fDistance < pMap->Sprite[i].HiTag)
		{
			float fPercent;
			if(nAmbiants[j] == 0)
			{
				nAmbiants[j] = XBUILD_SOUND_Play(c->DefineSound[GetCorrectSoundNumber(pMap->Sprite[i].LoTag)].strFilename, 1);
			}
			fPercent = ((float)pMap->Sprite[i].HiTag);
			fPercent = (fPercent - fDistance) / fPercent;
			fPercent *= 100.0f;
			XBUILD_SOUND_SetVolume(nAmbiants[j], (int)fPercent);
		}
		else
		{
			if(nAmbiants[j])
			{
				XBUILD_SOUND_Stop(nAmbiants[j]);
				nAmbiants[j] = 0;
			}
		}
	}
}

int CGame::GetCorrectSoundNumber(int Number)
{
	int i;
	for(i = 0; i < c->nDefineSoundCount; i++)
	{
		if(c->DefineSound[i].nNumber == Number)
		{
			return i;
		}
	}
	return 0;
}

void CGame::ProcessMap()
{
	XBUILD_MAP *m;
	int i;
	m = (XBUILD_MAP*)XBUILD_GFX_GetMapAddress();
	memset(nAmbiants, 0, sizeof(int) * 512);
	memset(nAmbiantsSprites, 0, sizeof(int) * 512);
	nAmbiantCount = 0;
	for(i = 0; i < m->Mapinfo.SpriteNumber; i++)
	{
		if(m->Sprite[i].Pic > 0 && m->Sprite[i].Pic < 11)
		{
			m->Sprite[i].CStat |= 0x8000;
		}
		if(m->Sprite[i].Palette == 1)
		{
			m->Sprite[i].CStat |= 0x8000;
		}
		if(m->Sprite[i].Pic == 1)
		{
			if(m->Sprite[i].LoTag == 13)
			{
				//Explosive Cans
				//TODO: Check angle for accuracy
				m->Sector[m->Sprite[i].SectorNum].FZ = m->Sector[m->Sprite[i].SectorNum].CZ;
			}
		}
		if(m->Sprite[i].Pic == 1405)
		{
			//Duke Sprites
			m->Sprite[i].CStat |= 0x8000;
		}
#ifndef DISPLAYENEMIES
		if(m->Sprite[i].Pic >= 1680 && m->Sprite[i].Pic <= 2120)
		{
			m->Sprite[i].CStat |= 0x8000;
		}
#endif
		if(m->Sprite[i].Pic == 5)
		{
			if(m->Sector[m->Sprite[i].SectorNum].LoTag == 0)
			{
				if(m->Sprite[i].LoTag < 1000)
				{
					nAmbiantsSprites[nAmbiantCount++] = i;
				}
			}
		}
	}
}

void CGame::CompileCONs()
{
	int nErrors;
	char *pEpisode4;
	unsigned _int32 dwE4;
	FILE *stream;
	c = new XBUILD_CONSCRIPT_CONTEXT;
	memset(c, 0, sizeof(XBUILD_CONSCRIPT_CONTEXT));
	if(!XBUILD_GRPMANAGER_GetFileData("E4L1.MAP", (unsigned _int8**)&pEpisode4, &dwE4))
	{
		XBUILD_CONSCRIPT_SetCompilerMode(XBUILD_COMPILER_MODE_V13);
	}
	else
	{
		XBUILD_CONSCRIPT_SetCompilerMode(XBUILD_COMPILER_MODE_V1415);
	}
	stream = fopen("CONDATA.DAT", "rb");
	if(stream == NULL)
	{
		nErrors = XBUILD_CONSCRIPT_Compile(strCON, c);
		XBUILD_CONSOLE_PrintFormatted("%i Errors.\r\n", nErrors);
		stream = fopen("CONDATA.DAT", "wb");
		fwrite(c, sizeof(XBUILD_CONSCRIPT_CONTEXT), 1, stream);
		fclose(stream);
		XBUILD_CONSOLE_PrintFormatted("Saved CON data to 'CONDATA.DAT'.\r\n");
	}
	else
	{
		XBUILD_CONSOLE_PrintFormatted("Loading CON data from 'CONDATA.DAT'.\r\n");
		fread(c, sizeof(XBUILD_CONSCRIPT_CONTEXT), 1, stream);
		fclose(stream);
	}
}

void CGame__SysComm(int wParam, void *pParam)
{
	CGame *g;
	g = (CGame*)pParam;
	if(wParam == 0xF060)
	{
		g->nQuit = 1;
	}
}

void CGame::BeginLoop()
{
	XBUILDDISPLAYELEMENT_FILTER Filter;
	CScriptManager sm;
	CCtrlGame CtrlGame;
	CCtrlConsole CtrlConsole;
	CMovie Movie;
	CModelDef mdl;
	XBUILD_MAP *pMap2;
	XBUILD_ART_TILEDATA d;
	char lpstrTemp[256];
	HWND hWnd;
	int i;
	XBUILD_MAP *pMapTemp;
	SetupMod();
	if(!XBUILD_Initialize(strGRP, "DUKE.RTS"))
	{
		return;
	}
	XBUILD_CONSOLE_PrintFormatted("Starting glDuke...\r\n");
	XBUILD_CONSOLE_ShowTextConsole(TRUE);
	CompileCONs();
	XBUILD_ART_Initialize();
	XBUILD_CONSOLE_HideTextConsole();
	XBUILD_GFX_SetStatus(NULL);
	hWnd = XBUILD_GFX_Initialize(set.m_Setup.lpstrRenderer);
	SetWindowText(hWnd, "glDuke");
	SendMessage(hWnd, WM_SETICON, 0, (LPARAM)LoadIcon(AfxGetApp()->m_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME)));
	if(set.m_Setup.nSoundEnabled)
	{
		XBUILD_SOUND_Initialize(hWnd);
	}
	XBUILD_ANIMATE_Initialize();
	XBUILD_INCREMENTATOR_Initialize();
	diw.Initialize();
	if(set.m_Setup.dwControlStatus & 0x01)
	{
		diw.InitializeKeyboard(hWnd);
	}
	if(set.m_Setup.dwControlStatus & 0x02)
	{
		diw.InitializeMouse(hWnd);
	}
	//Movie.PlayANM("CINEOV3.ANM");
	if(m_Level.Compare(""))
	{
		if(!IsBuiltInLevel())
		{
			nUserLevel = 1;
		}
	}
	else
	{
		m_Level.Format("%s", c->DefineLevelname[0].strFilename);
	}
	pMapTemp = new XBUILD_MAP;
	XBUILD_MAP_Load(m_Level.GetBuffer(256), pMapTemp);
	XBUILD_GFX_SetMap(pMapTemp);
	delete pMapTemp;
	pMap =		(XBUILD_MAP*)XBUILD_GFX_GetMapAddress();
	pMap2 =		pMap;
	X =			pMap->Mapinfo.StartX;
	Z =			pMap->Mapinfo.StartY;
	Y =			pMap->Mapinfo.StartZ;
	XAngle =	0;
	YAngle =	pMap->Mapinfo.StartA;

	mdl.LoadModelDefinitionFile(".\\MODELS\\MODELS.TXT");

	//XBUILD_GFX_AddTileToDisplayList(30, 5, 168, &m_Status, -1);
	//XBUILD_GFX_AddTileToDisplayList(31, 37, 168, &m_Status, -1);
	//XBUILD_GFX_AddTileToDisplayList(33, 69, 168, &m_Status, -1);
	//XBUILD_GFX_AddTileToDisplayList(-1, 5, 168, &m_Status, -1);
	XBUILD_GFX_AddTileToDisplayList(-1, 5, 168, &m_Status, -1);
	XBUILD_GFX_AddTileToDisplayList(-1, 5, 168, &m_Status, -1);

	//Crosshair centering code
	int nCX;
	int nCY;
	XBUILD_ART_GetTileData(2523, &d);
	nCX = (320 - d.Width) / 2;
	nCY = (200 - d.Height) / 2;

	XBUILD_GFX_AddTileToDisplayList(2523, nCX, nCY, &m_Status, -1);
//	XBUILD_GFX_AddTileToDisplayList(2613, 221, 120, &m_Status, -1);
	XBUILD_GFX_AddTileToDisplayList(2524, 221, 110, &m_Status, -1);
	XBUILD_GFX_AddTileToDisplayList(2462, 0, 167, &m_Status, -1);
	Filter.R = 0;
	Filter.G = 0;
	Filter.B = 0;
	Filter.A = 0;
	memcpy(m_Status.Element[STATUS_BASE_ELEMENT_COUNT - 1].pData, &Filter, sizeof(XBUILDDISPLAYELEMENT_FILTER));
	m_Status.Element[STATUS_BASE_ELEMENT_COUNT - 1].dwType = XBUILDDISPLAYLIST_TYPE_FILTER;

	m_Status.nElementCount = STATUS_BASE_ELEMENT_COUNT;
	m_Status.dwReferentialX = 320;
	m_Status.dwReferentialY = 200;

	ChangeHealth(c->Gamestartup.nMaxPlayerHealth);
	memset(nWeaponsAmmo, 0, 12 * 4);
	nWeaponsAmmo[1] = 50;

	for(i = 0; i < MAXSECTORS; i++)
	{
		SectorStatus[i].bActivated = 0;
		SectorStatus[i].nIncrementator = -1;
		SectorStatus[i].pGame = (void*)this;
		SectorStatus[i].nSector = i;
	}

	XBUILD_GFX_SetDisplayList(XBUILD_GFX_LAYER_STATUS, &m_Status);
	XBUILD_GFX_SetStatus(XBUILD_GFX_LAYER_3DMAP | XBUILD_GFX_LAYER_STATUS);
	XBUILD_CONSOLE_SetGraphicConsoleParams(DUKE_STATUSFONTBASE - 0x20, 200);
	XBUILD_GFX_SetSystemCommandCallback(CGame__SysComm, this);
	if(set.m_Setup.nMusicEnabled)
	{
		if(nUserLevel)
		{
			XBUILD_MUSIC_Play(c->Music[1].strFilename02, set.m_Setup.nMusicDriver);
		}
		else
		{
			PlayCorrectMusic();
		}
	}
	ProcessMap();
	sm.FireAllScripts(c, &X, &Z, &Y, this);
	pCtrlGame = (void*)&CtrlGame;
	pCtrlConsole = (void*)&CtrlConsole;
	CtrlGame.PassControlFocus(this);
	while(1)
	{
		switch(nControlFocus)
		{
		case CONTROL_FOCUS_GAME:
			CtrlGame.Update();
			break;
		case CONTROL_FOCUS_CONSOLE:
			CtrlConsole.Update();
			break;
		}
		if(nQuit == 1)
		{
			break;
		}
		XBUILD_ANIMATE_SetCurrentPosition(X, Z, Y);
		ProcessSectorTags();
		UpdateAmbientSounds();
		XBUILD_INCREMENTATOR_Update();
		sprintf(lpstrTemp, "glDuke : %i FPS", 1000 / XBUILD_GFX_Update(X, Y, Z, XAngle, YAngle));
		SetWindowText(hWnd, lpstrTemp);
	}
	sm.DeleteAll();
	delete c;
	XBUILD_INCREMENTATOR_Destroy();
	XBUILD_MUSIC_Stop();
	XBUILD_SOUND_Destroy();
	XBUILD_GFX_Destroy();
	XBUILD_ART_Destroy();
}
