#include "COMMON.H"

#define GETVALUE_TYPE_INTEGER	0x0001
#define GETVALUE_TYPE_STRING	0x0002

#define LONGCOMMENT			"/*"
#define COMMENT				"//"
#define DEFINE				"define"
#define DEFINEQUOTE			"definequote"
#define DEFINESOUND			"definesound"
#define DEFINEVOLUMENAME	"definevolumename"
#define DEFINESKILLNAME		"defineskillname"
#define DEFINELEVELNAME		"definelevelname"
#define	MUSIC				"music"
#define GAMESTARTUP			"gamestartup"
#define ACTOR				"actor"
#define USERACTOR			"useractor"
#define STATE				"state"
#define ENDA				"enda"
#define ENDS				"ends"
#define INCLUDE				"include"

#define COMPILER_MODE_V13		1
#define COMPILER_MODE_V1415		2

unsigned int					CONSCRIPT_nCompilerMode = 0;

#define ADDDEFINE(d, c)				memcpy(&c->Define[c->nDefineCount], &d, sizeof(BLK_DEFINE));									c->nDefineCount++;
#define ADDDEFINEQUOTE(d, c)		memcpy(&c->DefineQuote[c->nDefineQuoteCount], &d, sizeof(BLK_DEFINEQUOTE));					c->nDefineQuoteCount++;
#define ADDDEFINESOUND(d, c)		memcpy(&c->DefineSound[c->nDefineSoundCount], &d, sizeof(BLK_DEFINESOUND));					c->nDefineSoundCount++;
#define ADDDEFINEVOLUMENAME(d, c)	memcpy(&c->DefineVolumename[c->nDefineVolumenameCount], &d, sizeof(BLK_DEFINEVOLUMENAME));	c->nDefineVolumenameCount++;
#define ADDDEFINESKILLNAME(d, c)	memcpy(&c->DefineSkillname[c->nDefineSkillnameCount], &d, sizeof(BLK_DEFINESKILLNAME));		c->nDefineSkillnameCount++;
#define ADDDEFINELEVELNAME(d, c)	memcpy(&c->DefineLevelname[c->nDefineLevelnameCount], &d, sizeof(BLK_DEFINELEVELNAME));		c->nDefineLevelnameCount++;
#define ADDMUSIC(d, c)				memcpy(&c->Music[c->nMusicCount], &d, sizeof(BLK_MUSIC));									c->nMusicCount++;
#define ADDACTOR(d, c)				memcpy(&c->Actor[c->nActorCount], d, sizeof(BLK_ACTOR));									c->nActorCount++;
#define ADDSTATE(d, c)				memcpy(&c->State[c->nStateCount], d, sizeof(BLK_STATE));									c->nStateCount++;

__inline long CONSCRIPT_GetFileSize(FILE *lpStream)
{
	long nTemp;
	long nSize;
	nTemp = ftell(lpStream);
	fseek(lpStream, 0, SEEK_END);
	nSize = ftell(lpStream);
	fseek(lpStream, nTemp, SEEK_SET);
	return nSize;
}

int CONSCRIPT_GetNextString(char *lpOutput, FILE *lpStream)
{
	unsigned _int8 nByte;
	int nPos = 0;
	memset(lpOutput, 0, 256);
	if(ftell(lpStream) >= CONSCRIPT_GetFileSize(lpStream))
	{
		return 0;
	}
	while(ftell(lpStream) < CONSCRIPT_GetFileSize(lpStream))
	{
		fread(&nByte, 1, 1, lpStream);
		if(nByte == 0x0D)
		{
			//Line feed (except a carriage return)
			fseek(lpStream, 1, SEEK_CUR);
			fread(&nByte, 1, 1, lpStream);
			if(nByte == 0x00 || nByte == 0x20)
			{
				while(ftell(lpStream) < CONSCRIPT_GetFileSize(lpStream))
				{
					//Skip long spaces
					fread(&nByte, 1, 1, lpStream);
					if(nByte != 0x00 && nByte != 0x20 && nByte != 0x0A && nByte != 0x0D)
					{
						fseek(lpStream, -1, SEEK_CUR);
						break;
					}
				}
			}
			else
			{
				fseek(lpStream, -1, SEEK_CUR);
			}
			return 1;
		}
		if(nByte == 0x0A || nByte == 0x00 || nByte == 0x20)
		{
			//Either a space or a carriage return
			if(nByte == 0x00 || nByte == 0x20)
			{
				while(ftell(lpStream) < CONSCRIPT_GetFileSize(lpStream))
				{
					//Skip long spaces
					fread(&nByte, 1, 1, lpStream);
					if(nByte != 0x00 && nByte != 0x20 && nByte != 0x0A && nByte != 0x0D)
					{
						fseek(lpStream, -1, SEEK_CUR);
						break;
					}
				}
			}
			return 1;
		}
		lpOutput[nPos] = nByte;
		nPos++;
	}
	return 1;
}

int CONSCRIPT_GetUntilString(char *lpOutput, char *lpString, FILE *lpStream)
{
	char lpBuff[256];
	int nStrSize = strlen(lpString);
	int nPos = 0;
	while(ftell(lpStream) < CONSCRIPT_GetFileSize(lpStream))
	{
		memset(lpBuff, 0, 256);
		fread(lpBuff, nStrSize, 1, lpStream);
		if(!strnicmp(lpBuff, lpString, nStrSize))
		{
			break;
		}
		fseek(lpStream, 1 - nStrSize, SEEK_CUR);
		lpOutput[nPos] = lpBuff[0];
		nPos++;
	}
	return nPos;
}

int CONSCRIPT_GetNumber(char *lpString, XBUILD_CONSCRIPT_CONTEXT *c)
{
	int nValue;
	int i;
	for(i = 0; i < c->nDefineCount; i++)
	{
		if(!strnicmp(lpString, c->Define[i].strSymbol, strlen(lpString)))
		{
			return c->Define[i].nValue;
		}
	}
	sscanf(lpString, "%i", &nValue);
	return nValue;
}

__inline void CONSCRIPT_GetValue(void *lpPtr, int nType, int nSize, FILE *lpStream, XBUILD_CONSCRIPT_CONTEXT *c)
{
	char lpString[256];
	CONSCRIPT_GetNextString(lpString, lpStream);
	switch(nType)
	{
	case GETVALUE_TYPE_INTEGER:
		*(int*)(lpPtr) = CONSCRIPT_GetNumber(lpString, c);
		break;
	case GETVALUE_TYPE_STRING:
		memcpy(lpPtr, lpString, nSize);
		break;
	}
}

void CONSCRIPT_GetUntilLine(char *lpOutput, FILE *lpStream)
{
	unsigned _int8 nByte;
	int nPos = 0;
	memset(lpOutput, 0, 256);
	while(ftell(lpStream) < CONSCRIPT_GetFileSize(lpStream))
	{
		fread(&nByte, 1, 1, lpStream);
		if(nByte == 0x0D)
		{
			//Line feed (except a carriage return)
			fseek(lpStream, 1, SEEK_CUR);
			return;
		}
		if(nByte == 0x0A)
		{
			//Either a space or a carriage return
			return;
		}
		lpOutput[nPos] = nByte;
		nPos++;
	}
}

void CONSCRIPT_SkipLongComment(FILE *lpStream)
{
	unsigned _int16 nWord;
	while(ftell(lpStream) < CONSCRIPT_GetFileSize(lpStream))
	{
		fread(&nWord, 2, 1, lpStream);
		if(nWord == 0x2F2A)
		{
			return;
		}
		fseek(lpStream, -1, SEEK_CUR);
	}
}

extern "C" void DLLEXPORT XBUILD_CONSCRIPT_SetCompilerMode(int nMode)
{
	CONSCRIPT_nCompilerMode	= nMode;
}

extern "C" int DLLEXPORT XBUILD_CONSCRIPT_Compile(char *lpstrFile, XBUILD_CONSCRIPT_CONTEXT *c)
{
	char lpString[256];
	int nError = 0;
	FILE *lpStream;
	XBUILD_CONSOLE_PrintFormatted("Compiling '%s'...\r\n", lpstrFile);
	XBUILD_GRPMANAGER_ExtractFile(lpstrFile, lpstrFile);
	lpStream = fopen(lpstrFile, "rb");
	if(lpStream == NULL)
	{
		return -1;
	}
	while(CONSCRIPT_GetNextString(lpString, lpStream) == 1)
	{
		if(!strcmp(lpString, INCLUDE))
		{
			CONSCRIPT_GetValue(lpString, GETVALUE_TYPE_STRING, 256, lpStream, c);
			nError += XBUILD_CONSCRIPT_Compile(lpString, c);
		}
		else if(!strcmp(lpString, LONGCOMMENT))
		{
			CONSCRIPT_SkipLongComment(lpStream);
		}
		else if(!strcmp(lpString, COMMENT))
		{
			CONSCRIPT_GetUntilLine(lpString, lpStream);
		}
		else if(!strcmp(lpString, DEFINE))
		{
			BLK_DEFINE def;
			CONSCRIPT_GetValue(def.strSymbol, GETVALUE_TYPE_STRING, 64, lpStream, c);
			CONSCRIPT_GetValue(&def.nValue, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			ADDDEFINE(def, c);
		}
		else if(!strcmp(lpString, DEFINEQUOTE))
		{
			BLK_DEFINEQUOTE defq;
			CONSCRIPT_GetValue(&defq.nNumber, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetUntilLine(lpString, lpStream);
			memcpy(defq.strQuote, lpString, 64);
			ADDDEFINEQUOTE(defq, c);
		}
		else if(!strcmp(lpString, DEFINESOUND))
		{
			BLK_DEFINESOUND defs;
			CONSCRIPT_GetValue(&defs.nNumber, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(defs.strFilename, GETVALUE_TYPE_STRING, 32, lpStream, c);
			CONSCRIPT_GetValue(&defs.nPitchVariation1, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defs.nPitchVariation2, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defs.nPriority, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defs.nStatus, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defs.nVolumeAdjust, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			ADDDEFINESOUND(defs, c);
		}
		else if(!strcmp(lpString, DEFINEVOLUMENAME))
		{
			BLK_DEFINEVOLUMENAME defvn;
			CONSCRIPT_GetValue(&defvn.nNumber, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetUntilLine(lpString, lpStream);
			memcpy(defvn.strName, lpString, 32);
			ADDDEFINEVOLUMENAME(defvn, c);
		}
		else if(!strcmp(lpString, DEFINESKILLNAME))
		{
			BLK_DEFINESKILLNAME defsn;
			CONSCRIPT_GetValue(&defsn.nNumber, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetUntilLine(lpString, lpStream);
			memcpy(defsn.strName, lpString, 32);
			ADDDEFINESKILLNAME(defsn, c);
		}
		else if(!strcmp(lpString, DEFINELEVELNAME))
		{
			BLK_DEFINELEVELNAME defln;
			CONSCRIPT_GetValue(&defln.nEpisode, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defln.nLevel, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(defln.strFilename, GETVALUE_TYPE_STRING, 32, lpStream, c);
			CONSCRIPT_GetValue(defln.strPar, GETVALUE_TYPE_STRING, 10, lpStream, c);
			CONSCRIPT_GetValue(defln.str3DRealmsPar, GETVALUE_TYPE_STRING, 10, lpStream, c);
			CONSCRIPT_GetUntilLine(lpString, lpStream);
			memcpy(defln.strLevelname, lpString, 32);
			ADDDEFINELEVELNAME(defln, c);
		}
		else if(!strcmp(lpString, MUSIC))
		{
			BLK_MUSIC defm;
			CONSCRIPT_GetValue(&defm.nEpisode, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(defm.strFilename01, GETVALUE_TYPE_STRING, 32, lpStream, c);
			CONSCRIPT_GetValue(defm.strFilename02, GETVALUE_TYPE_STRING, 32, lpStream, c);
			if(defm.nEpisode != 0)
			{
				CONSCRIPT_GetValue(defm.strFilename03, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename04, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename05, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename06, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename07, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename08, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename09, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename10, GETVALUE_TYPE_STRING, 32, lpStream, c);
				CONSCRIPT_GetValue(defm.strFilename11, GETVALUE_TYPE_STRING, 32, lpStream, c);
			}
			ADDMUSIC(defm, c);
		}
		else if(!strcmp(lpString, GAMESTARTUP))
		{
			BLK_GAMESTARTUP defg;
			if(CONSCRIPT_nCompilerMode == COMPILER_MODE_V1415)
			{
				CONSCRIPT_GetValue(&defg.nVisibility,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nImpactDamage,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxPlayerHealth,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nArmorHealth,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nActorRespawnTime,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nItemRespawnTime,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nRunningSpeed,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nGravitationnalConstant,	GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nRPGBlastRadius,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nPipebombBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nShrinkerBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nTripbombBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMorterBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nBounceMineBlastRadius,	GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nSeenineBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxPistolAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxShotgunAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxChaingunAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxRPGAmmo,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxPipebombAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxShrinkerAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxDevastatorAmmo,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxTripbombAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxFreezeAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxGrowAmmo,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nCameraDestructible,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nNumberFreezeBounces,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nFreezerHurtsOwner,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nQSize,					GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nTripbombLaserMode,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				memcpy(&c->Gamestartup, &defg, sizeof(BLK_GAMESTARTUP));
				c->nGamestartupCount = 1;
			}
			else
			{
				BLK_GAMESTARTUP defg;
				CONSCRIPT_GetValue(&defg.nVisibility,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nImpactDamage,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxPlayerHealth,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nArmorHealth,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nActorRespawnTime,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nItemRespawnTime,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nRunningSpeed,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nGravitationnalConstant,	GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nRPGBlastRadius,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nPipebombBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nShrinkerBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nTripbombBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMorterBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nBounceMineBlastRadius,	GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nSeenineBlastRadius,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxPistolAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxShotgunAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxChaingunAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxRPGAmmo,				GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxPipebombAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxShrinkerAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxDevastatorAmmo,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxTripbombAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nMaxFreezeAmmo,			GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nCameraDestructible,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nNumberFreezeBounces,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				CONSCRIPT_GetValue(&defg.nFreezerHurtsOwner,		GETVALUE_TYPE_INTEGER, 0, lpStream, c);
				memcpy(&c->Gamestartup, &defg, sizeof(BLK_GAMESTARTUP));
				c->nGamestartupCount = 1;
			}
		}
		else if(!strcmp(lpString, ACTOR))
		{
			BLK_ACTOR *defa;
			defa = new BLK_ACTOR;
			memset(defa, 0, sizeof(BLK_ACTOR));
			CONSCRIPT_GetValue(&defa->nSprite, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defa->nHealth, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetUntilString(defa->strCode, ENDA, lpStream);
			ADDACTOR(defa, c);
			if(c->nActorCount > MAXACTOR)
			{
				MessageBox(NULL, "TOO MANY ACTORS!", "BLAH!", 16);
			}
			delete defa;
		}
		else if(!strcmp(lpString, USERACTOR))
		{
			BLK_ACTOR *defa;
			defa = new BLK_ACTOR;
			memset(defa, 0, sizeof(BLK_ACTOR));
			//First should be "association" ie: enemy or notenemy
			CONSCRIPT_GetValue(&defa->nSprite, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			/////////////////////////////////////////////////////
			CONSCRIPT_GetValue(&defa->nSprite, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetValue(&defa->nHealth, GETVALUE_TYPE_INTEGER, 0, lpStream, c);
			CONSCRIPT_GetUntilString(defa->strCode, ENDA, lpStream);
			ADDACTOR(defa, c);
			if(c->nActorCount > MAXACTOR)
			{
				MessageBox(NULL, "TOO MANY ACTORS!", "BLAH!", 16);
			}
			delete defa;
		}
		else if(!strcmp(lpString, STATE))
		{
			BLK_STATE *defs;
			defs = new BLK_STATE;
			memset(defs, 0, sizeof(BLK_STATE));
			CONSCRIPT_GetValue(&defs->strName, GETVALUE_TYPE_STRING, 256, lpStream, c);
			CONSCRIPT_GetUntilString(defs->strCode, ENDS, lpStream);
			ADDSTATE(defs, c);
			delete defs;
		}
		else if(!strlen(lpString))
		{
			//Empty string
		}
		else
		{
			nError++;
		}
	}
	fclose(lpStream);
	DeleteFile(lpstrFile);
	return nError;
}
